package com.fundi.framework.common.displaylist;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.bind.annotation.XmlRootElement;

import com.fundi.framework.common.nl1.Messages;

@XmlRootElement
public class ListFilterCondition implements Serializable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	public static final String opEquals = Messages.getString("ListFilterCondition_4"); //$NON-NLS-1$
	public static final String opNotEquals = Messages.getString("ListFilterCondition_5"); //$NON-NLS-1$
	public static final String opLess = Messages.getString("ListFilterCondition_6"); //$NON-NLS-1$
	public static final String opLessEqual = Messages.getString("ListFilterCondition_7"); //$NON-NLS-1$
	public static final String opGreater = Messages.getString("ListFilterCondition_8"); //$NON-NLS-1$
	public static final String opGreaterEqual = Messages.getString("ListFilterCondition_9"); //$NON-NLS-1$
	public static final String opIn = Messages.getString("ListFilterCondition_10"); //$NON-NLS-1$
	public static final String opNotIn = Messages.getString("ListFilterCondition_11"); //$NON-NLS-1$
	public static final String opBetween = Messages.getString("ListFilterCondition_12"); //$NON-NLS-1$
	public static final String opContains = Messages.getString("ListFilterCondition_21"); //$NON-NLS-1$
	public static final String opWildcardSingle = Messages.getString("ListFilterCondition_13"); //$NON-NLS-1$
	public static final String opWildcardMulti = Messages.getString("ListFilterCondition_14"); //$NON-NLS-1$
	public static final String opWildcardSingleRegex = Messages.getString("ListFilterCondition_15"); //$NON-NLS-1$
	public static final String opWildcardMultiRegex = Messages.getString("ListFilterCondition_16"); //$NON-NLS-1$
	public static final String[] allowedOperands = {opIn, opNotIn, opBetween, opEquals, opNotEquals, opLess,
																			opLessEqual, opGreater, opGreaterEqual, opContains};
	
	public static final String anyField = Messages.getString("ListFilterCondition_22"); //$NON-NLS-1$
	
	private String[] values;
	private int totalValues = 0;
	private Pattern[] pattern;
	boolean singleValueTest = false;
	boolean numericTest = false;
	boolean wildcardTest = false;
	
	private ListFieldDefn fieldDefn = null;
	private String operator = ""; //$NON-NLS-1$
	private String uneditedValue = ""; //$NON-NLS-1$
	private boolean highlight = false;
	private int red = 0;
	private int green = 0;
	private int blue = 0;
	
	private boolean caseInsensitive = false;  //Used the negative for compatibility with filters saved before this field was added.
	
	public ListFilterCondition() {
		
		super();
	}
	
	public ListFilterCondition(ListFieldDefn fieldDefn, String operator, String uneditedValue, boolean highlight, int red, int green, int blue,
											boolean caseSensitive) {
		
		this.fieldDefn = fieldDefn;
		this.operator = operator;
		this.uneditedValue = uneditedValue;
		this.highlight = highlight;
		this.red = red;
		this.green = green;
		this.blue = blue;
		this.caseInsensitive = !caseSensitive;

		extractMultipleValues();
	}

	public void extractMultipleValues() {
		
		StringTokenizer parser = null;
		String separator;
		int cnt;
		
		if (operator.equals(opIn) || operator.equals(opNotIn) || operator.equals(opBetween)) {
			if (operator.equals(opIn)|| operator.equals(opNotIn)) {
				separator = Messages.getString("ListFilterCondition_19"); //$NON-NLS-1$
			}
			else {
				separator = Messages.getString("ListFilterCondition_20"); //$NON-NLS-1$
			}	
			parser = new StringTokenizer(uneditedValue, separator);
			totalValues = parser.countTokens();
		}
		else {
			totalValues = 1;
		}
		
		values = new String[totalValues];
		if (parser == null) {
			values[0] = uneditedValue;
		}
		else {
			cnt = -1;
			try {
				while (parser.hasMoreTokens()) {
					values[++cnt] = parser.nextToken();
				}
			}
			catch (NoSuchElementException nsee) {}
		}
		
		if (operator.equals(opEquals) || operator.equals(opNotEquals) || operator.equals(opLess) || operator.equals(opLessEqual) ||
				operator.equals(opGreater) || operator.equals(opGreaterEqual) || operator.equals(opContains)) {
			singleValueTest = true;
		}
		if (fieldDefn.getDataType().equals(ListFieldDefn.typeNumeric)) {
			numericTest = true;
		}
		else {
			if (operator.equals(opIn) || operator.equals(opNotIn) || operator.equals(opEquals) || operator.equals(opNotEquals)) {
				for (cnt = 0; cnt < totalValues; cnt++) {
					if (((String) values[cnt]).indexOf(opWildcardSingle) >= 0 || ((String) values[cnt]).indexOf(opWildcardMulti) >= 0) {
						wildcardTest = true;
						break;
					}
				}
			}
			//ICON users are used to * and % wildcard, but the Java Regex Pattern equivalents are .* and . respectively.
			if (wildcardTest) {
				pattern = new Pattern[totalValues];
				for (cnt = 0; cnt < totalValues; cnt++) {
					values[cnt] = ((String) values[cnt]).replace(opWildcardSingle, opWildcardSingleRegex);
					values[cnt] = ((String) values[cnt]).replace(opWildcardMulti, opWildcardMultiRegex);
					if (isCaseInsensitive()) {
						pattern[cnt] = Pattern.compile(((String) values[cnt]).toLowerCase());
					}
					else {
						pattern[cnt] = Pattern.compile((String) values[cnt]);
					}
				}
			}
		}
	}
	
	public boolean valueMeetsCondition(ListField field) {
		
		return valueMeetsCondition(field, false);
	}
	
	public boolean valueMeetsCondition(ListField field, boolean forceNumericTest) {
		
		boolean met = false;
		int comparison1 = 0;
		int comparison2 = 0;
		int cnt;
		Matcher matcher;
		String testValue = null;
		
		if (field == null) {
			return false;
		}
		else {
			 testValue = field.getValue();
		}
		
		if (pattern == null) {
			extractMultipleValues();
		}
		
		/*
		 * If it's a filtering condition and the field name is not for this condition, then exit saying the condition is met (i.e. can be ignored satisfactorily).
		 * 
		 * If it's a highlight condition and the field name is not for this condition, then exit saying the condition is not met (i.e. no highlight).
		 */
		if (!fieldDefn.getLabel().equals(anyField)) {
			if (!(field.getDefn().getLabel().equals(fieldDefn.getLabel()))) {
				if (isHighlight()) {
					return false;
				}
				else {
					return true;
				}
			}
		}
		
		 if (numericTest || forceNumericTest) {
			 if (testValue.length() == 0) {
				 return false;  //Can't satisfy the condition if it's an empty string in the field.
			 }
		 }
		
		if (singleValueTest && !wildcardTest) {
			if (numericTest || forceNumericTest) {
				comparison1 = (new Double(testValue)).compareTo(new Double(values[0]));
			}
			else {
				if (operator.equals(opContains)) {
					if (isCaseInsensitive()) {
						met = ((String) testValue).toLowerCase().contains(((String) values[0]).toLowerCase());
					}
					else {
						met = ((String) testValue).contains((String) values[0]);
					}	
					return met;
				}
				
				if (isCaseInsensitive()) {
					comparison1 = ((String) testValue).toLowerCase().compareTo(((String) values[0]).toLowerCase());
				}
				else {
					comparison1 = ((String) testValue).compareTo((String) values[0]);
				}
			}
			if (comparison1 == 0 && (operator.indexOf(opEquals) >= 0 || operator.equals(opNotEquals))) {
				met = true;
			}
			else {
				if (comparison1 < 0 && (operator.equals(opLess) || operator.equals(opLessEqual))) {
					met = true;
				}
				else {
					if (comparison1 > 0 && (operator.equals(opGreater) || operator.equals(opGreaterEqual))) {
						met = true;
					}
				}
			}
			if (operator.equals(opNotEquals)) {
				met = !met;
			}
			return met;
		}

		if (!singleValueTest && !wildcardTest) {
			if (operator.equals(opIn) || operator.equals(opNotIn)) {
				for (cnt = 0; cnt < totalValues; cnt++) {
					if (numericTest || forceNumericTest) {
						comparison1 = (new Double(testValue)).compareTo(new Double(values[cnt]));
					}
					else {
						if (isCaseInsensitive()) {
							comparison1 = ((String) testValue).toLowerCase().compareTo(((String) values[cnt]).toLowerCase());
						}
						else {
							comparison1 = ((String) testValue).compareTo((String) values[cnt]);
						}
					}
					if (comparison1 == 0) {
						met = true;
						if (operator.equals(opIn)) {
							break;
						}
					}
				}
				if ((operator.equals(opNotIn) || operator.equals(opNotEquals)) && met) {
					met = false;
				}
				else {
					if ((operator.equals(opNotIn) || operator.equals(opNotEquals)) && !met) {
						met = true;
					}
				}
				return met;
			}
			else {
				if (numericTest || forceNumericTest) {
					comparison1 = (new Double(testValue)).compareTo(new Double(values[0]));
					comparison2 = (new Double(testValue)).compareTo(new Double(values[1]));
				}
				else {
					if (isCaseInsensitive()) {
						comparison1 = ((String) testValue).toLowerCase().compareTo(((String) values[0]).toLowerCase());
						comparison2 = ((String) testValue).toLowerCase().compareTo(((String) values[1]).toLowerCase());
					}
					else {
						comparison1 = ((String) testValue).compareTo((String) values[0]);
						comparison2 = ((String) testValue).compareTo((String) values[1]);
					}
				}
				if (comparison1 >= 0 && comparison2 <= 0) {
					met = true;
				}
				return met;
			}
		}
		
		//Only a wildcard test is left to consider.
		for (cnt = 0; cnt < totalValues; cnt++) {
			if (isCaseInsensitive()) {
				matcher = pattern[cnt].matcher(((String) testValue).toLowerCase());
			}
			else {
				matcher = pattern[cnt].matcher((String) testValue);
			}
			if (matcher.matches()) {
				met = true;
				if (operator.equals(opIn) || operator.equals(opEquals)) {
					break;
				}
			}
		}
		if ((operator.equals(opNotIn) || operator.equals(opNotEquals)) && met) {
			met = false;
		}
		else {
			if ((operator.equals(opNotIn) || operator.equals(opNotEquals)) && !met) {
				met = true;
			}
		}
		return met;
	}

	public ListFieldDefn getFieldDefn() {
		return fieldDefn;
	}

	public void setFieldDefn(ListFieldDefn fieldDefn) {
		this.fieldDefn = fieldDefn;
	}

	public String getOperator() {
		return operator;
	}

	public void setOperator(String operator) {
		this.operator = operator;
	}

	public String getUneditedValue() {
		return uneditedValue;
	}

	public void setUneditedValue(String uneditedValue) {
		this.uneditedValue = uneditedValue;
	}

	public boolean isHighlight() {
		return highlight;
	}

	public void setHighlight(boolean highlight) {
		this.highlight = highlight;
	}

	public int getRed() {
		return red;
	}

	public void setRed(int red) {
		this.red = red;
	}

	public int getGreen() {
		return green;
	}

	public void setGreen(int green) {
		this.green = green;
	}

	public int getBlue() {
		return blue;
	}

	public void setBlue(int blue) {
		this.blue = blue;
	}

	public String[] getValues() {
		return values;
	}

	public void setValues(String[] values) {
		this.values = values;
	}

	public int getTotalValues() {
		return totalValues;
	}

	public void setTotalValues(int totalValues) {
		this.totalValues = totalValues;
	}

	public boolean isSingleValueTest() {
		return singleValueTest;
	}

	public void setSingleValueTest(boolean singleValueTest) {
		this.singleValueTest = singleValueTest;
	}

	public boolean isNumericTest() {
		return numericTest;
	}

	public void setNumericTest(boolean numericTest) {
		this.numericTest = numericTest;
	}

	public boolean isWildcardTest() {
		return wildcardTest;
	}

	public void setWildcardTest(boolean wildcardTest) {
		this.wildcardTest = wildcardTest;
	}

	public boolean isCaseInsensitive() {
		return caseInsensitive;
	}

	public void setCaseInsensitive(boolean caseInsensitive) {
		this.caseInsensitive = caseInsensitive;
	}
	
	public String toString() {
		StringBuilder	result;
		
		result = new StringBuilder();
		result.append(fieldDefn.getLabel());
		result.append(" ");
		result.append(operator);
		result.append(" ");
		if (opIn.equals(operator) || opNotIn.equals(operator) || opBetween.equals(operator)) {
			result.append("(");
			result.append(uneditedValue);
			result.append(")");
		} else { 
			result.append(uneditedValue);
		}
		
		return result.toString();
	}
	
	public boolean equals(Object o) {
		boolean 			result;
		ListFilterCondition	other;
		
		result = false;
		if (ListFilterCondition.class.isInstance(o)) {
			other = (ListFilterCondition) o;
			result = this.fieldDefn.getLabel().equals(other.fieldDefn.getLabel()) && this.operator.equals(other.operator) && this.uneditedValue.equals(other.uneditedValue);
		}
		
		return result;
	}
}
