package com.fundi.framework.common.displaylist;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;
import java.util.ArrayList;

import javax.xml.bind.annotation.XmlRootElement;

import com.fundi.framework.common.cslcomms.CSLField;
import com.fundi.framework.common.properties.ISelectionSource;
import com.fundi.framework.common.properties.PropertyDefn;
import com.fundi.framework.common.util.StringEditor;

@XmlRootElement
public class ListRow implements Comparable<ListRow>, Serializable, ISelectionSource {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private ArrayList<ListField> fields = new ArrayList<ListField>();
	private DisplayList list = null;  //The holding list.
	
	private int uid;  //Unique identifier of the row, generated by the DisplayList class.
	private Object data = null;  //The source object for this row.
	private boolean visible = true;
	
	private String dataSource = ""; //$NON-NLS-1$
	
	public static final int stringPadLength = 50;
	public static final int numericPadLength = 20;
	
	private ListFieldColor highlightColor = new ListFieldColor();
	
	public ListRow() {
		
		super();
	}
	
	public void addField(ListField field) {
		
		/*
		 * A field can only exist once in the row.
		 * 
		 * The row may be being built from multiple sources, so check the field isn't already defined for the row.
		 */
		
		int idxFld;
		boolean fnd = false;
		
		for (idxFld = 0; idxFld < fields.size(); idxFld++) {
			if (fields.get(idxFld).getLabel().equalsIgnoreCase(field.getLabel())) {
				fnd = true;
				break;
			}
		}
		
		if (!fnd) {
			fields.add(field);
		}
	}

	public void clearFields() {
		
		fields.clear();
	}
	
	public ListField getField(String label) {
		
		int idxFld;
		ListField field;
		
		for (idxFld = 0; idxFld < fields.size(); idxFld++) {
			field = fields.get(idxFld);
			if (list == null || !list.compareRows()) {
				if (field.getLabel().equalsIgnoreCase(label)) {
					return field;
				}
			}
			else {
				if (field.getCompareLabel().equalsIgnoreCase(label)) {
					return field;
				}
			}
		}
		
		return null;
	}
	
	public ArrayList<ListField> getFields() {
		
		return fields;
	}
	
	public ListField getListFieldFor(PropDefn pdefn) {
		ListField	result;
		
		result = null;
		for (ListField field: fields) {
			if (field.getDefn().getKeyLabel().equals(pdefn.getKeyLabel())) {
				result = field;
				break;
			}
		}
		
		return result;
	}

	public void setList(DisplayList list) {
		
		this.list = list;
	}

	@Override
	public int compareTo(ListRow o) {

		int comparison;
		
		comparison = getSortValue().compareTo(o.getSortValue());
		
		if (!(list.isAscending())) {
			comparison = -comparison;
		}
		
		return comparison;
	}
	
	public String getSortValue() {
		
		//The sort value is the key value plus the data source value.
		
		StringBuffer sortVal = new StringBuffer(""); //$NON-NLS-1$
		StringEditor edt;
		
		if (list != null) {
			sortVal = new StringBuffer(getKeyValue());
			
			//Append the data source, so the row can be used for comparator purposes.
			edt = new StringEditor(dataSource);
			sortVal.append(edt.pad(stringPadLength, " ", true)); //$NON-NLS-1$
		}

		return sortVal.toString().toLowerCase();
	}
	
	public String getKeyValue() {
		
		//Return the concatenation of the key fields.  Left pad integers with zeroes, right pad strings with spaces.
		//An arbitrary maximum length of 50 characters is used for strings, 20 for integers.
		
		StringBuffer keyVal = new StringBuffer(""); //$NON-NLS-1$
		int idxKey;
		ArrayList<String> keyFields;
		ListField field;
		StringEditor edt;
		String value;
		
		if (list != null) {
			keyFields = list.getKeyFields();
			
			for (idxKey = 0; idxKey < keyFields.size(); idxKey++) {
				field = getField(keyFields.get(idxKey));
				if (field == null) {
					keyVal.append(new StringEditor(" ").pad(stringPadLength, " ", true)); //$NON-NLS-1$ //$NON-NLS-2$
				}
				else {
					value = field.getValue();
					if (value != null) {
						edt = new StringEditor(field.getValue());
						if (field.getDataType().equals(ListFieldDefn.typeCharacter)) {
							keyVal.append(edt.pad(stringPadLength, " ", true)); //$NON-NLS-1$
						}
						else {
							if (field.getDataType().equals(ListFieldDefn.typeNumeric)) {			
								keyVal.append(edt.pad(numericPadLength, "0", false)); //$NON-NLS-1$
							}
						}
					}
					else {
						keyVal.append(new StringEditor(" ").pad(stringPadLength, " ", true)); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			}
		}

		return keyVal.toString();
	}
	
	public String getHashValue() {
		
		//Return the concatenation of all the visible field values.  Left pad integers with zeroes, right pad strings with spaces.
		//An arbitrary maximum length of 50 characters is used for strings, 20 for integers.
		
		StringBuffer hashVal = new StringBuffer(""); //$NON-NLS-1$
		int idxKey;
		ArrayList<ListFieldDefn> fields;
		ListField field;
		StringEditor edt;
		String value;
		
		if (list != null) {
			fields = list.getFieldDefns();
			
			for (idxKey = 0; idxKey < fields.size(); idxKey++) {
				field = getField(fields.get(idxKey).getLabel());
				if (field != null) {
					if (field.isVisible()) {
						value = field.getValue();
						if (value != null) {
							edt = new StringEditor(field.getValue());
							if (field.getDataType().equals(ListFieldDefn.typeCharacter)) {
								hashVal.append(edt.pad(stringPadLength, " ", true)); //$NON-NLS-1$
							}
							else {
								if (field.getDataType().equals(ListFieldDefn.typeNumeric)) {			
									hashVal.append(edt.pad(numericPadLength, "0", false)); //$NON-NLS-1$
								}
							}
						}
						else {
							hashVal.append(new StringEditor(" ").pad(stringPadLength, " ", true)); //$NON-NLS-1$ //$NON-NLS-2$
						}
					}
				}
			}
		}

		return hashVal.toString();
	}

	public String getDataSource() {
		return dataSource;
	}

	public void setDataSource(String dataSource) {
		this.dataSource = dataSource;
	}

	public boolean isBlankRow() {
		
		return (fields.size() <= 0);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((getHashValue() == null) ? 0 : getHashValue().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ListRow other = (ListRow) obj;
		if (getHashValue() == null) {
			if (other.getHashValue() != null)
				return false;
		} else if (!getHashValue().equals(other.getHashValue()))
			return false;
		return true;
	}

	public Object getData() {
		
		return data;
	}

	public void setData(Object data) {
		
		this.data = data;
	}

	public int getUid() {
		
		return uid;
	}

	public void setUid(int uid) {
		
		this.uid = uid;
	}

	public boolean isVisible() {
		
		return visible;
	}

	public void setVisible(boolean visible) {
		
		this.visible = visible;
	}

	public ListFieldColor getHighlightColor() {
		return highlightColor;
	}

	public void setHighlightColor(ListFieldColor highlightColor) {
		this.highlightColor = highlightColor;
	}
	
	public void resetHighlightColor() {
		
		if (highlightColor == null) {
			highlightColor = new ListFieldColor();
		}
		else {
			highlightColor.reset();
		}
	}

	public DisplayList getList() {
		
		return list;
	}

	@Override
	public PropertyDefn[] getPropertyDefns() {
		
		int idx;
		int idxShow;
		PropertyDefn[] defns;

		if (ISelectionSource.class.isInstance(getData())) {
			defns = ((ISelectionSource) getData()).getPropertyDefns();
		}
		else {
			idxShow = 0;
			for (idx = 0; idx < fields.size(); idx++) {
				if (fields.get(idx).isVisible()) {
					idxShow++;
				}
			}
			defns = new PropertyDefn[idxShow];
			idxShow = -1;
			for (idx = 0; idx < fields.size(); idx++) {
				if (fields.get(idx).isVisible()) {
					defns[++idxShow] = new PropertyDefn(fields.get(idx).getLabel(), fields.get(idx).getLabel());
				}
			}
		}

		return defns;
	}

	@Override
	
	public String getPropertyValue(String id) {

		if (ISelectionSource.class.isInstance(getData())) {
			return ((ISelectionSource) getData()).getPropertyValue(id);
		}
		else {
			return getField(id).getValue();
		}
	}

	@Override
	public String toString() {
		StringBuilder	sb = new StringBuilder("ListRow "); //$NON-NLS-1$

		sb.append(uid);
		sb.append(": "); //$NON-NLS-1$
		sb.append(getFields().toString());
		
		return sb.toString();
	}

	public ListField getFieldByKeyLabel(String keyLabel) {
		ListField result;
		
		result = null;
		for (ListField fld: getFields()) {	//MUST use accessor here - so that in the situation where you're in an EListRow you retrieve src.fields not this.fields!!!
			if (fld.getKeyLabel().equals(keyLabel)) {
				result = fld;
				break;
			}
		}
		
		return result;
	}
	
	public String getFieldValueByKeyLabel(String keyLabel) {
		ListField resultField;
		
		resultField = getFieldByKeyLabel(keyLabel);
		
		return (resultField == null ? null : resultField.getValue()); 
	}
}
